{
  GridView
  Copyright (C) 1996-2002 by Berg
  All rights reserved.

  $id:ColumnEditor.pas 01/04/2003 22:46:35 bn
}

unit ColumnEditor;

interface

uses
	{$IFDEF VER130}DsgnIntf, DsgnWnds,{$ELSE}DesignWindows, DesignIntf,{$ENDIF}	Windows,
  Classes, Controls, ComCtrls, GridView, ImgList, SysUtils, Graphics, CustomGridView, Forms,
  Columns, CustomGridViewControl, ToolWin, ScrollView, CommCtrl;

type
  TColumnForm = class(TDesignWindow)
    DesignerGrid: TGridView;
    ToolBar1: TToolBar;
    btnTextual: TToolButton;
    btnImage: TToolButton;
    imlToolbar: TImageList;
    btnProgress: TToolButton;
    btnCheckBox: TToolButton;
    ToolButton6: TToolButton;
    btnDelete: TToolButton;
    btnClear: TToolButton;
    btnUp: TToolButton;
    btnDown: TToolButton;
    ToolButton8: TToolButton;
    imlImages: TImageList;
    btnIncrement: TToolButton;
    btnRate: TToolButton;
    btnList: TToolButton;
    btnNumeric: TToolButton;
    ToolButton1: TToolButton;
    procedure FormShow(Sender: TObject);
    procedure btnTextualClick(Sender: TObject);
    procedure btnImageClick(Sender: TObject);
    procedure btnProgressClick(Sender: TObject);
    procedure btnCheckBoxClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure DesignerGridCustomDrawCell(Sender: TObject; ACol,
      ARow: Integer; CellRect: TRect; CellState: TCellState);
    procedure btnIncrementClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnRateClick(Sender: TObject);
    procedure DesignerGridSelectCell(Sender: TObject; ACol, ARow: Integer);
    procedure btnListClick(Sender: TObject);
    procedure btnNumericClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
  private
    FOwner: TPersistent;
		{$IFDEF VER130}
    FDesigner: IFormDesigner;
		{$ELSE}
    FDesigner: IDesigner;
		{$ENDIF}
    FEditingColumn: TCustomColumn;
    procedure AddColumn(ColumnKind: TColumnKind);
    procedure RefreshList;
    procedure SelectColumns;
    procedure SelectLastColumn;
    procedure SetOwner(const Value: TPersistent);
    procedure SetEditingColumn(const Value: TCustomColumn);
  protected
  	function GetColumnClass(ColumnKind: TColumnKind): TColumnClass; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  	procedure UpdateColumnInEditor(Column: TCustomColumn);
		{$IFDEF VER130}
    property Designer: IFormDesigner read FDesigner write FDesigner;
		{$ELSE}
    property Designer: IDesigner read FDesigner write FDesigner;
		{$ENDIF}
    property Owner: TPersistent read FOwner write SetOwner;
    property EditingColumn: TCustomColumn read FEditingColumn write SetEditingColumn;
  end;

implementation

uses
  ColumnClasses;

{$R *.dfm}

{ TColumnForm }

procedure TColumnForm.AddColumn(ColumnKind: TColumnKind);
begin
  TGridView(FOwner).Columns.Add(GetColumnClass(ColumnKind));
  RefreshList;
  SelectLastColumn;
  Designer.Modified;
end;

procedure TColumnForm.RefreshList;
var
  i: Integer;
  Grid: TGridView;
begin
  Grid := TGridView(FOwner);
  DesignerGrid.ClearRows;
  for i := 0 to Grid.Columns.Count - 1 do
  begin
    DesignerGrid.AddRow;
    if Grid.Columns[i] is TTextualColumn then DesignerGrid.Cells[0, i] := '0' else
    if Grid.Columns[i] is TImageColumn then DesignerGrid.Cells[0, i] := '1' else
    if Grid.Columns[i] is TProgressColumn then DesignerGrid.Cells[0, i] := '2' else
    if Grid.Columns[i] is TCheckBoxColumn then DesignerGrid.Cells[0, i] := '3' else
    if Grid.Columns[i] is TIncrementColumn then DesignerGrid.Cells[0, i] := '4' else
    if Grid.Columns[i] is TRateColumn then DesignerGrid.Cells[0, i] := '5';
    if Grid.Columns[i] is TListColumn then DesignerGrid.Cells[0, i] := '6';
    if Grid.Columns[i] is TNumericColumn then DesignerGrid.Cells[0, i] := '7';
    if Grid.Columns[i] is TDateColumn then DesignerGrid.Cells[0, i] := '8';

    if coFixedSize in Grid.Columns[i].Options
      then DesignerGrid.Cells[1, i] := '-1' else
    begin
      if coAutoSize in Grid.Columns[i].Options
        then DesignerGrid.Cells[1, i] := '10'
          else DesignerGrid.Cells[1, i] := '9';
    end;
    DesignerGrid.Cells[2, i] := '';
    DesignerGrid.Cells[3, i] := IntToStr(i) + ': ' + Grid.Columns[i].Header.Caption;
    DesignerGrid.Cells[4, i] := Grid.Columns[i].ClassName;
  end;
  if DesignerGrid.RowCount > 0 then DesignerGrid.SelectedRow := DesignerGrid.RowCount - 1;
end;

procedure TColumnForm.SelectLastColumn;
begin
  with FOwner as TGridView do
    Designer.SelectComponent(Columns[Columns.Count - 1]);
end;

function TColumnForm.GetColumnClass(ColumnKind: TColumnKind): TColumnClass;
begin
  case ColumnKind of
    ckTextual: Result := TTextualColumn;
    ckNumeric: Result := TNumericColumn;
    ckImage: Result := TImageColumn;
    ckList: Result := TListColumn;
    ckCheckBox: Result := TCheckBoxColumn;
    ckDate: Result := TDateColumn;
    ckProgress: Result := TProgressColumn;
    ckRate: Result := TRateColumn;
    ckIncrement: Result := TIncrementColumn;
    else Result := TCustomColumn;
  end;
end;

procedure TColumnForm.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  { we need to free (and close) this window
    when FOwner (component) has been deleted }
  if (Operation = opRemove) and (AComponent = FOwner) then Free;
end;

procedure TColumnForm.SetEditingColumn(const Value: TCustomColumn);
begin
  FEditingColumn := Value;
end;

procedure TColumnForm.SetOwner(const Value: TPersistent);
begin
  FOwner := Value;
end;

  { Events }

procedure TColumnForm.FormShow(Sender: TObject);
begin
  { note: next line notify window (Self)
          that FOwner will be destroyed }
  TGridView(FOwner).FreeNotification(Self);
  RefreshList;
  if DesignerGrid.RowCount > 0 then DesignerGrid.SelectedRow := 0;
end;

procedure TColumnForm.btnTextualClick(Sender: TObject);
begin
  AddColumn(ckTextual);
end;

procedure TColumnForm.btnImageClick(Sender: TObject);
begin
  AddColumn(ckImage);
end;

procedure TColumnForm.btnProgressClick(Sender: TObject);
begin
  AddColumn(ckProgress);
end;

procedure TColumnForm.btnCheckBoxClick(Sender: TObject);
begin
  AddColumn(ckCheckBox);
end;

procedure TColumnForm.btnDeleteClick(Sender: TObject);
var
  i: Integer;
begin
  Designer.Modified;
  if DesignerGrid.RowCount = 0 then Exit;
  { Note: Can't delete column because this control is
    showed in Object Inspector ! }
  Designer.NoSelection;

  i := 0;
  while i < DesignerGrid.RowCount do { delete all selected columns }
  begin
    if DesignerGrid.Selected[i] then
    begin
      DesignerGrid.DeleteRow(i);
		  TGridView(FOwner).Columns.Delete(i);
    end else Inc(i);
  end;
  if (DesignerGrid.SelectedRow >= DesignerGrid.RowCount) and (DesignerGrid.RowCount > 0)
  	then DesignerGrid.SelectedRow := DesignerGrid.RowCount - 1;
end;

procedure TColumnForm.btnClearClick(Sender: TObject);
begin
  Designer.NoSelection;
  TGridView(FOwner).Columns.Clear;
  DesignerGrid.ClearRows;
  Designer.Modified;
end;

procedure TColumnForm.btnUpClick(Sender: TObject);
begin
  DesignerGrid.MoveRow(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
  with FOwner as TGridView do
  begin
	  Columns.Move(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
	  Columns.ChangePosition(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow - 1);
  end;
  DesignerGrid.SelectedRow := DesignerGrid.SelectedRow - 1;
  Designer.Modified;
end;

procedure TColumnForm.btnDownClick(Sender: TObject);
begin
  DesignerGrid.MoveRow(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
  with FOwner as TGridView do
  begin
  	Columns.Move(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
  	Columns.ChangePosition(DesignerGrid.SelectedRow, DesignerGrid.SelectedRow + 1);
	end;
  DesignerGrid.SelectedRow := DesignerGrid.SelectedRow + 1;
  Designer.Modified;
end;

procedure TColumnForm.DesignerGridCustomDrawCell(Sender: TObject; ACol,
  ARow: Integer; CellRect: TRect; CellState: TCellState);
var
  SmallRect: TRect;
begin
  if ACol = 2 then
  begin
    SmallRect := CellRect;
    InflateRect(SmallRect, -4, -2);
    DesignerGrid.Canvas.Pen.Color := clGrayText;
    DesignerGrid.Canvas.Brush.Color := TGridView(FOwner).Columns[ARow].Color;
    DesignerGrid.Canvas.Rectangle(SmallRect);
  end;
end;

procedure TColumnForm.btnIncrementClick(Sender: TObject);
begin
  AddColumn(ckIncrement);
end;

procedure TColumnForm.FormDestroy(Sender: TObject);
begin
  TGridView(FOwner).RemoveFreeNotification(Self);
end;

procedure TColumnForm.btnRateClick(Sender: TObject);
begin
  AddColumn(ckRate);
end;

procedure TColumnForm.SelectColumns;
var
  CompList: IDesignerSelections;
  ColumnPersistent: TPersistent;
  i: Integer;
begin
  CompList := CreateSelectionList;
  for i := 0 to DesignerGrid.RowCount - 1 do
  	if DesignerGrid.Selected[i] then
		begin
      ColumnPersistent := TGridView(FOwner).Columns[i];
{$IFDEF VER130}
			CompList.Add(MakeIPersistent(ColumnPersistent));
{$ELSE}
			CompList.Add(ColumnPersistent);
{$ENDIF}
	  end;
  if CompList.Count > 0 then
	begin
    EditingColumn := TGridView(FOwner).Columns[DesignerGrid.SelectedRow];
	  Designer.SetSelections(CompList);
	end;
end;

procedure TColumnForm.DesignerGridSelectCell(Sender: TObject; ACol,
  ARow: Integer);
begin
  btnUp.Enabled := ARow > 0;
  btnDown.Enabled := ARow < DesignerGrid.RowCount - 1;
  if DesignerGrid.RowCount = 0 then Exit;
  SelectColumns;
end;

procedure TColumnForm.btnListClick(Sender: TObject);
begin
  AddColumn(ckList);
end;

procedure TColumnForm.UpdateColumnInEditor(Column: TCustomColumn);
begin
  DesignerGrid.Cells[3, Column.Index] := IntToStr(Column.Index) + ': ' + Column.Header.Caption;
end;

procedure TColumnForm.btnNumericClick(Sender: TObject);
begin
  AddColumn(ckNumeric);
end;

procedure TColumnForm.FormActivate(Sender: TObject);
begin
  ActiveControl := DesignerGrid;
  SelectColumns;
end;

procedure TColumnForm.ToolButton1Click(Sender: TObject);
begin
  AddColumn(ckDate);
end;

end.

