{
  CellSource 1.0
  Copyright (C) 1996-2002 by Berg
  All rights reserved.

  $id:DataCellSource.pas
}

unit DataCellSource;

interface

uses
  Windows, Messages, SysUtils, Classes,
  DB, GridView, CustomCellSource;

type
  TDataCellSource = class(TCustomCellSource)
  private
    FDataSet: TDataSet;
    procedure AddColumns;
    procedure SetDataSet(const Value: TDataSet);
  public
    function Execute: Boolean; override;
    function IsReady: Boolean; override;
    function Update: Boolean; override;
  published
    property DataSet: TDataSet read FDataSet write SetDataSet;
  end;

procedure Register;

implementation

uses
  Columns, ColumnClasses, FileCellSource;

procedure Register;
begin
  RegisterComponents('Berg', [TDataCellSource]);
end;

{ TDataCellSource }

procedure TDataCellSource.AddColumns;
var
  i: Integer;
begin
  if Grid.Columns.Count = 0 then
    for i := 0 to FDataSet.FieldCount - 1 do
    begin
      case FDataSet.Fields[i].DataType of
        ftBoolean:
          begin
            Grid.Columns.Add(TCheckBoxColumn);
            TCheckBoxColumn(Grid.Columns[i]).CheckedText := 'True';
            TCheckBoxColumn(Grid.Columns[i]).UncheckedText := 'False';
          end;
        else Grid.Columns.Add(TTextualColumn);
      end;
    end;
end;

procedure TDataCellSource.SetDataSet(const Value: TDataSet);
begin
  FDataSet := Value;
end;

function TDataCellSource.Execute;
var
  i, CurrFld, CurrCol: Integer;
begin
  if not inherited Execute then Exit;
  AddColumns;
  Grid.BeginUpdate;
  Grid.ClearRows;
	try
    try
      FDataSet.First;
      for i := 1 to FDataSet.RecordCount do
      begin
        CurrCol := 0;
        CurrFld := 0;
        Grid.AddRow;
        while (CurrCol < Grid.Columns.Count) and (CurrFld < FDataSet.FieldCount) do
        begin
          while not (coPublicUsing in Grid.Columns[CurrCol].Options) do
          begin
            Inc(CurrCol);
            if CurrCol >= Grid.Columns.Count then Break;
          end;
          if Grid.Columns.Count > CurrCol then
          begin
            Grid.Cells[CurrCol, i - 1] := FDataSet.Fields[CurrFld].AsString;
          end;
          Inc(CurrFld);
          Inc(CurrCol);
        end;
        FDataSet.Next;
      end;
    except
      Result := False;
      Exit;
    end;
  finally
	  Grid.EndUpdate;
  end;
  Result := True;
end;

function TDataCellSource.IsReady: Boolean;
begin
  Result := False;
  if Assigned(Grid) then
    if Assigned(FDataSet) then
      if not DataSet.IsUnidirectional then
        if FDataSet.Active then Result := True;
end;

function TDataCellSource.Update: Boolean;
var
  i, CurrFld, CurrCol: Integer;
begin
  if not inherited Execute then Exit;
  Grid.BeginUpdate;
	try
    try
      FDataSet.First;
      for i := 1 to FDataSet.RecordCount do
      begin
        CurrCol := 0;
        CurrFld := 0;
        if Grid.RowCount < i then Grid.AddRow;
        while (CurrCol < Grid.Columns.Count) and (CurrFld < FDataSet.FieldCount) do
        begin
          while not (coPublicUsing in Grid.Columns[CurrCol].Options) do
          begin
            Inc(CurrCol);
            if CurrCol >= Grid.Columns.Count then Break;
          end;
          if (Grid.Columns.Count > CurrCol)
          	and (Grid.Cells[CurrCol, i - 1] <> FDataSet.Fields[CurrFld].AsString)
            	then
          begin

            Grid.Cells[CurrCol, i - 1] := FDataSet.Fields[CurrFld].AsString;
          end;
          Inc(CurrFld);
          Inc(CurrCol);
        end;
        FDataSet.Next;
      end;
    except
      Result := False;
      Exit;
    end;
  finally
	  Grid.EndUpdate;
  end;
  Result := True;
end;

end.
